@@ -13,14 +13,13 @@
 ############################################################################
 """Component and Component Architecture Interfaces
 
-$Id: interfaces.py 88794 2008-07-24 16:17:58Z wichert $
+$Id: interfaces.py 105528 2009-11-08 11:05:28Z kobold $
 """
+__docformat__ = "reStructuredText"
 
-import zope.deferredimport
-from zope import interface
-
-# BBB: Backward-compatibility; 12/05/2004
-from bbb.interfaces import *
+from zope.interface import Attribute
+from zope.interface import Interface
+from zope.interface import implements
 
 class ComponentLookupError(LookupError):
     """A component could not be found."""
@@ -32,27 +31,23 @@ class Misused(Exception):
     """A component is being used (registered) for the wrong interface."""
 
 
-class IObjectEvent(interface.Interface):
+class IObjectEvent(Interface):
     """An event related to an object.
 
     The object that generated this event is not necessarily the object
     refered to by location.
     """
 
-    object = interface.Attribute("The subject of the event.")
+    object = Attribute("The subject of the event.")
 
 
 class ObjectEvent(object):
-    interface.implements(IObjectEvent)
-
-    # for repr backward compatibility. In the next release cycle, we'll
-    # provide a testing framework that addresses repr migration.
-    __module__ = 'zope.app.event.objectevent'
+    implements(IObjectEvent)
 
     def __init__(self, object):
         self.object = object
 
-class IComponentArchitecture(interface.Interface):
+class IComponentArchitecture(Interface):
     """The Component Architecture is defined by two key components: Adapters
     and Utiltities. Both are managed by site managers. All other components
     build on top of them.
@@ -94,6 +89,20 @@ class IComponentArchitecture(interface.Interface):
         the specified interface.  If one is not found, returns default.
         """
 
+    def queryNextUtility(context, interface, name='', default=None):
+        """Query for the next available utility.
+    
+        Find the next available utility providing `interface` and having the
+        specified name. If no utility was found, return the specified `default`
+        value.
+        """
+    
+    def getNextUtility(context, interface, name=''):
+        """Get the next available utility.
+    
+        If no utility was found, a `ComponentLookupError` is raised.
+        """
+
     def getUtilitiesFor(interface, context=None):
         """Return the utilities that provide an interface
 
@@ -112,7 +121,7 @@ class IComponentArchitecture(interface.Interface):
     # Adapter API
 
     def getAdapter(object,
-                   interface=interface.Interface, name=u'',
+                   interface=Interface, name=u'',
                    context=None):
         """Get a named adapter to an interface for an object
 
@@ -149,7 +158,7 @@ class IComponentArchitecture(interface.Interface):
         """
 
     def getMultiAdapter(objects,
-                        interface=interface.Interface, name='',
+                        interface=Interface, name='',
                         context=None):
         """Look for a multi-adapter to an interface for an objects
 
@@ -167,7 +176,7 @@ class IComponentArchitecture(interface.Interface):
         named adapter methods with an empty string for a name.
         """
 
-    def queryAdapter(object, interface=interface.Interface, name=u'',
+    def queryAdapter(object, interface=Interface, name=u'',
                      default=None, context=None):
         """Look for a named adapter to an interface for an object
 
@@ -204,7 +213,7 @@ class IComponentArchitecture(interface.Interface):
         """
 
     def queryMultiAdapter(objects,
-                          interface=interface.Interface, name=u'',
+                          interface=Interface, name=u'',
                           default=None,
                           context=None):
         """Look for a multi-adapter to an interface for objects
@@ -298,17 +307,17 @@ class IComponentArchitecture(interface.Interface):
         create objects which implement the given interface.
         """
 
-class IComponentLookup(interface.Interface):
+class IComponentLookup(Interface):
     """Component Manager for a Site
 
     This object manages the components registered at a particular site. The
     definition of a site is intentionally vague.
     """
 
-    adapters = interface.Attribute(
+    adapters = Attribute(
         "Adapter Registry to manage all registered adapters.")
 
-    utilities = interface.Attribute(
+    utilities = Attribute(
         "Adapter Registry to manage all registered utilities.")
 
     def queryAdapter(object, interface, name=u'', default=None):
@@ -379,12 +388,7 @@ class IComponentLookup(interface.Interface):
         returned.
         """
 
-zope.deferredimport.deprecated(
-    "Use IComponentLookup instead.  ISiteManager will be removed in 2007.",
-    ISiteManager = "zope.component.interfaces:IComponentLookup",
-    )
-
-class IComponentRegistrationConvenience(interface.Interface):
+class IComponentRegistrationConvenience(Interface):
     """API for registering components.
 
     CAUTION: This API should only be used from test or
@@ -466,7 +470,7 @@ class IComponentRegistrationConvenience(interface.Interface):
         activity.
         """
 
-class IRegistry(interface.Interface):
+class IRegistry(Interface):
     """Object that supports component registry
     """
 
@@ -474,12 +478,12 @@ class IRegistry(interface.Interface):
         """Return an iterable of component registrations
         """
 
-class IFactory(interface.Interface):
+class IFactory(Interface):
     """A factory is responsible for creating other components."""
 
-    title = interface.Attribute("The factory title.")
+    title = Attribute("The factory title.")
 
-    description = interface.Attribute("A brief description of the factory.")
+    description = Attribute("A brief description of the factory.")
 
     def __call__(*args, **kw):
         """Return an instance of the objects we're a factory for."""
@@ -493,15 +497,15 @@ class IFactory(interface.Interface):
         instance cannot be created, an empty Implements instance is returned.
         """
 
-class IRegistration(interface.Interface):
+class IRegistration(Interface):
     """A registration-information object
     """
 
-    registry = interface.Attribute("The registry having the registration")
+    registry = Attribute("The registry having the registration")
 
-    name = interface.Attribute("The registration name")
+    name = Attribute("The registration name")
 
-    info = interface.Attribute("""Information about the registration
+    info = Attribute("""Information about the registration
 
     This is information deemed useful to people browsing the
     configuration of a system. It could, for example, include
@@ -512,24 +516,24 @@ class IUtilityRegistration(IRegistration):
     """Information about the registration of a utility
     """
 
-    factory = interface.Attribute("The factory used to create the utility. Optional.")
-    component = interface.Attribute("The object registered")
-    provided = interface.Attribute("The interface provided by the component")
+    factory = Attribute("The factory used to create the utility. Optional.")
+    component = Attribute("The object registered")
+    provided = Attribute("The interface provided by the component")
 
 class _IBaseAdapterRegistration(IRegistration):
     """Information about the registration of an adapter
     """
 
-    factory = interface.Attribute("The factory used to create adapters")
+    factory = Attribute("The factory used to create adapters")
 
-    required = interface.Attribute("""The adapted interfaces
+    required = Attribute("""The adapted interfaces
 
     This is a sequence of interfaces adapters by the registered
     factory.  The factory will be caled with a sequence of objects, as
     positional arguments, that provide these interfaces.
     """)
 
-    provided = interface.Attribute("""The interface provided by the adapters.
+    provided = Attribute("""The interface provided by the adapters.
 
     This interface is implemented by the factory
     """)
@@ -544,9 +548,9 @@ class ISubscriptionAdapterRegistration(_IBaseAdapterRegistration):
 
 class IHandlerRegistration(IRegistration):
 
-    handler = interface.Attribute("An object called used to handle an event")
+    handler = Attribute("An object called used to handle an event")
 
-    required = interface.Attribute("""The handled interfaces
+    required = Attribute("""The handled interfaces
 
     This is a sequence of interfaces handled by the registered
     handler.  The handler will be caled with a sequence of objects, as
@@ -559,7 +563,7 @@ class IRegistrationEvent(IObjectEvent):
 class RegistrationEvent(ObjectEvent):
     """There has been a change in a registration
     """
-    interface.implements(IRegistrationEvent)
+    implements(IRegistrationEvent)
 
     def __repr__(self):
         return "%s event:\n%r" % (self.__class__.__name__, self.object)
@@ -569,7 +573,7 @@ class IRegistered(IRegistrationEvent):
     """
 
 class Registered(RegistrationEvent):
-    interface.implements(IRegistered)
+    implements(IRegistered)
 
 class IUnregistered(IRegistrationEvent):
     """A component or factory was unregistered
@@ -578,9 +582,9 @@ class IUnregistered(IRegistrationEvent):
 class Unregistered(RegistrationEvent):
     """A component or factory was unregistered
     """
-    interface.implements(IUnregistered)
+    implements(IUnregistered)
 
-class IComponentRegistry(interface.Interface):
+class IComponentRegistry(Interface):
     """Register components
     """
 
@@ -919,3 +923,22 @@ class IComponentRegistry(interface.Interface):
 class IComponents(IComponentLookup, IComponentRegistry):
     """Component registration and access
     """
+
+
+class IPossibleSite(Interface):
+    """An object that could be a site.
+    """
+
+    def setSiteManager(sitemanager):
+        """Sets the site manager for this object.
+        """
+
+    def getSiteManager():
+        """Returns the site manager contained in this object.
+
+        If there isn't a site manager, raise a component lookup.
+        """
+
+
+class ISite(IPossibleSite):
+    """Marker interface to indicate that we have a site"""
